@@ -84,6 +84,14 @@
 #'   [0.9.6](https://r-spatial.github.io/sf/news/index.html#version-0-9-6-2020-09-13),
 #'    if `quiet` is equal to `FALSE`, then vectortranslate operations will
 #'   display a progress bar.
+#' @param boundary An `sf` or `sfc` object that will be used to create a spatial
+#'   filter during the vectortranslate operations. The type of filter can be
+#'   chosen using the argument `boundary_type`.
+#' @param boundary_type A character vector of length 1 specifying the type of
+#'   spatial filter. The `spat` filter selects only those features that
+#'   intersect a given area, while `clipsrc` also clips the geometries. See the
+#'   examples and check [here](https://gdal.org/programs/ogr2ogr.html) for more
+#'   details.
 #' @param download_only Boolean. If `TRUE`, then the function only returns the
 #'   path where the matched file is stored, instead of reading it. `FALSE` by
 #'   default.
@@ -106,53 +114,82 @@
 #'   and [`oe_read()`].
 #'
 #' @examples
-#' # Download OSM extracts associated to a simple test.
-#' its = oe_get("ITS Leeds", quiet = FALSE)
+#' # Match, download and convert OSM extracts associated to a simple test.
+#' its = oe_get("ITS Leeds", quiet = FALSE, download_directory = tempdir())
 #' class(its)
 #' unique(sf::st_geometry_type(its))
 #'
-#' # Get another layer from the test extract
+#' # Get another layer from ITS Leeds extract
 #' its_points = oe_get("ITS Leeds", layer = "points")
 #' unique(sf::st_geometry_type(its_points))
 #'
-#' # Get the .osm.pbf and .gpkg file path
-#' oe_get("ITS Leeds", download_only = TRUE)
-#' oe_get("ITS Leeds", download_only = TRUE, skip_vectortranslate = TRUE)
+#' # Get the .osm.pbf and .gpkg files paths
+#' oe_get("ITS Leeds", download_only = TRUE, quiet = TRUE)
+#' oe_get("ITS Leeds", download_only = TRUE, skip_vectortranslate = TRUE, quiet = TRUE)
 #' # See also ?oe_find()
 #'
 #' # Add additional tags
-#' its_with_oneway = oe_get("ITS Leeds", extra_tags = "oneway", quiet = FALSE)
+#' its_with_oneway = oe_get("ITS Leeds", extra_tags = "oneway")
 #' names(its_with_oneway)
 #' table(its_with_oneway$oneway, useNA = "ifany")
 #'
 #' # Use the query argument to get only oneway streets:
-#' q = "SELECT * FROM 'lines' WHERE oneway IN ('yes')"
+#' q = "SELECT * FROM 'lines' WHERE oneway == 'yes'"
 #' its_oneway = oe_get("ITS Leeds", query = q)
-#' its_oneway
+#' its_oneway[, c(1, 3, 9)]
 #'
+#' # Apply a spatial filter during the vectortranslate operations
+#' its_poly = sf::st_sfc(
+#'   sf::st_polygon(
+#'     list(rbind(
+#'       c(-1.55577, 53.80850),
+#'       c(-1.55787, 53.80926),
+#'       c(-1.56096, 53.80891),
+#'       c(-1.56096, 53.80736),
+#'       c(-1.55675, 53.80658),
+#'       c(-1.55495, 53.80749),
+#'       c(-1.55577, 53.80850)
+#'     ))
+#'   ),
+#'   crs = 4326
+#' )
+#' its_spat = oe_get("ITS Leeds", boundary = its_poly)
+#' its_clipped = oe_get("ITS Leeds", boundary = its_poly, boundary_type = "clipsrc", quiet = TRUE)
+#'
+#' plot(sf::st_geometry(its), reset = FALSE, col = "lightgrey")
+#' plot(sf::st_boundary(its_poly), col = "black", add = TRUE)
+#' plot(sf::st_boundary(sf::st_as_sfc(sf::st_bbox(its_poly))), col = "black", add = TRUE)
+#' plot(sf::st_geometry(its_spat), add = TRUE, col = "darkred")
+#' plot(sf::st_geometry(its_clipped), add = TRUE, col = "orange")
+#'
+#' # More complex examples
 #' \dontrun{
-#' # A more complex example
-#' west_yorkshire = oe_get("West Yorkshire", quiet = FALSE)
-#' # If you run it again, the function will not download the file
-#' # or convert it again
-#' west_yorkshire = oe_get("West Yorkshire", quiet = FALSE)
-#' # Match with place name
-#' oe_get("Milan") # Warning: the .pbf file is 400MB
-#' oe_get("Vatican City")
-#' oe_get("Zurich")
+#'   west_yorkshire = oe_get("West Yorkshire")
+#'   # If you run it again, the function will not download the file
+#'   # or convert it again
+#'   west_yorkshire = oe_get("West Yorkshire")
+#'   # Match with place name
+#'   oe_get("Milan") # Warning: the .pbf file is 400MB
+#'   oe_get("Vatican City") # Check all providers
+#'   oe_get("Zurich") # Use Nominatim API for geolocating places
 #'
-#' # Match with coordinates (any EPSG)
-#' milan_duomo = sf::st_sfc(sf::st_point(c(1514924, 5034552)), crs = 3003)
-#' oe_get(milan_duomo, quiet = FALSE) # Warning: the .pbf file is 400MB
-#' # Match with numeric coordinates (EPSG = 4326)
-#' oe_match(c(9.1916, 45.4650), quiet = FALSE)
-#' # Alternative providers
-#' baku = oe_get(place = "Baku", provider = "bbbike", quiet = FALSE)
+#'   # Match with coordinates (any EPSG)
+#'   milan_duomo = sf::st_sfc(sf::st_point(c(1514924, 5034552)), crs = 3003)
+#'   oe_get(milan_duomo, quiet = FALSE) # Warning: the .pbf file is 400MB
+#'   # Match with numeric coordinates (EPSG = 4326)
+#'   oe_match(c(9.1916, 45.4650), quiet = FALSE)
 #'
-#' # Other examples:
-#' oe_get("RU", match_by = "iso3166_1_alpha2", quiet = FALSE)
-#' # The following example mimics read_sf
-#' oe_get("Andora", stringsAsFactors = FALSE, quiet = TRUE, as_tibble = TRUE)}
+#'   # Check also alternative providers
+#'   baku = oe_get(place = "Baku")
+#'
+#'   # Other examples:
+#'   oe_get("RU", match_by = "iso3166_1_alpha2", quiet = FALSE)
+#'   # The following example mimics read_sf
+#'   oe_get("Andora", stringsAsFactors = FALSE, quiet = TRUE, as_tibble = TRUE)}
+#'
+#' # Remove .pbf and .gpkg files in tempdir
+#' # (since they may interact with other examples)
+#' file.remove(list.files(path = tempdir(), pattern = "(pbf|gpkg)", full.names = TRUE))
 oe_get = function(
   place,
   layer = "lines",
@@ -168,6 +205,8 @@ oe_get = function(
   osmconf_ini = NULL,
   extra_tags = NULL,
   force_vectortranslate = FALSE,
+  boundary = NULL,
+  boundary_type = c("spat", "clipsrc"),
   download_only = FALSE,
   skip_vectortranslate = FALSE,
   never_skip_vectortranslate = FALSE,
@@ -212,6 +251,8 @@ oe_get = function(
     extra_tags = extra_tags,
     force_vectortranslate = force_vectortranslate,
     never_skip_vectortranslate = never_skip_vectortranslate,
+    boundary = boundary,
+    boundary_type = boundary_type,
     quiet = quiet,
     ...
   )
