@@ -4,11 +4,10 @@ knitr::opts_chunk$set(
   comment = "#>", 
   fig.align = "center"
 )
-Sys.setenv(OSMEXT_DOWNLOAD_DIRECTORY = "")
 
 # save user's options and pars
-user_options <- options()
-user_par <- par(no.readonly = TRUE)
+user_options = options()
+user_par = par(no.readonly = TRUE)
 
 # set new options
 options(width = 100)
@@ -40,19 +39,19 @@ knitr::include_graphics(
 )
 
 ## -------------------------------------------------------------------------------------------------
-oe_match("Italy", quiet = TRUE)
-oe_match("Leeds", provider = "bbbike", quiet = TRUE)
+oe_match("Italy")
+oe_match("Leeds", provider = "bbbike")
 
 ## -------------------------------------------------------------------------------------------------
 oe_match("RU", match_by = "iso3166_1_alpha2")
 oe_match("US", match_by = "iso3166_1_alpha2")
 
 ## ---- error = TRUE--------------------------------------------------------------------------------
-oe_match("PS", match_by = "iso3166_1_alpha2")
-oe_match("IL", match_by = "iso3166_1_alpha2")
+oe_match("PS", match_by = "iso3166_1_alpha2", quiet = TRUE)
+oe_match("IL", match_by = "iso3166_1_alpha2", quiet = TRUE)
 
 ## -------------------------------------------------------------------------------------------------
-oe_match_pattern("London", provider = "geofabrik")
+oe_match_pattern("London")
 oe_match_pattern("Yorkshire")
 oe_match_pattern("Russia")
 oe_match_pattern("Palestine")
@@ -61,36 +60,32 @@ oe_match_pattern("Palestine")
 oe_match_pattern("US", match_by = "iso3166_2")
 
 ## -------------------------------------------------------------------------------------------------
-oe_match_pattern("Washington", provider = "bbbike")
+lapply(oe_match_pattern("Israel", full_row = TRUE), function(x) x[, 1:3])
 
 ## -------------------------------------------------------------------------------------------------
-oe_match_pattern("Israel", full_row = TRUE)[, 1:3]
-
-## -------------------------------------------------------------------------------------------------
-oe_match_pattern("London")
-oe_match("Greater London")
+oe_match_pattern("Valencia")
+oe_match("Comunitat Valenciana", provider = "openstreetmap_fr")
 
 ## ---- error = TRUE--------------------------------------------------------------------------------
 # erroneous match
-oe_match("London", max_string_dist = 3) 
+oe_match("Milan", max_string_dist = 2)
 
 ## -------------------------------------------------------------------------------------------------
-oe_match("leeds", provider = "geofabrik")
+oe_match("Leeds")
+oe_match("London")
+oe_match("Vatican City")
 
 ## ---- eval = FALSE--------------------------------------------------------------------------------
 #  oe_match("Milan")
 #  #> No exact match found for place = Milan and provider = geofabrik. Best match is Iran.
 #  #> Checking the other providers.
 #  #> No exact match found in any OSM provider data. Searching for the location online.
-#  #> .... (other messages explained below)
+#  #> The input place was matched with Nord-Ovest.
 #  #> $url
 #  #> [1] "https://download.geofabrik.de/europe/italy/nord-ovest-latest.osm.pbf"
 #  #> $file_size
 #  #> [1] 416306623
 
-## ---- warning = TRUE------------------------------------------------------------------------------
-oe_match("Belin")
-
 ## -------------------------------------------------------------------------------------------------
 milan_duomo = sf::st_sfc(sf::st_point(c(1514924, 5034552)), crs = 3003)
 oe_match(milan_duomo)
@@ -123,19 +118,19 @@ oe_match(c(9.1916, 45.4650)) # Duomo di Milano using EPSG: 4326
 
 ## -------------------------------------------------------------------------------------------------
 # ITS stands for Institute for Transport Studies: https://environment.leeds.ac.uk/transport
-(its_details = oe_match("ITS Leeds", provider = "test"))
+(its_details = oe_match("ITS Leeds"))
 
 ## ---- eval = FALSE--------------------------------------------------------------------------------
 #  oe_download(
 #    file_url = its_details$url,
 #    file_size = its_details$file_size,
 #    provider = "test",
-#    download_directory = "."
+#    download_directory = # path-to-a-directory
 #  )
 
 ## ---- eval = FALSE--------------------------------------------------------------------------------
 #  usethis::edit_r_environ()
-#  # Add a line containing: OSMEXT_DOWNLOAD_DIRECTORY=/path/to/save/files
+#  # Add a line containing: OSMEXT_DOWNLOAD_DIRECTORY=/path/for/osm/data
 
 ## -------------------------------------------------------------------------------------------------
 oe_download_directory()
@@ -161,6 +156,9 @@ st_layers(its_gpkg, do_count = TRUE)
 ## -------------------------------------------------------------------------------------------------
 oe_get_keys(its_gpkg, layer = "lines")
 
+## -------------------------------------------------------------------------------------------------
+oe_get_keys(its_gpkg, layer = "lines", values = TRUE)
+
 ## -------------------------------------------------------------------------------------------------
 its_gpkg = oe_vectortranslate(its_pbf, extra_tags = c("bicycle", "foot"))
 
@@ -187,13 +185,16 @@ plot(its_lines["highway"], lwd = 2, key.pos = NULL)
 #  oe_match("RU", match_by = "iso3166_1_alpha2", quiet = FALSE)
 #  
 #  oe_get("Andorra", download_only = TRUE)
-#  oe_get_keys(oe_get("Andorra", download_only = TRUE))
+#  oe_get_keys("Andorra")
+#  oe_get_keys("Andorra", values = TRUE)
+#  oe_get_keys("Andorra", values = TRUE, which_keys = c("oneway", "surface", "maxspeed"))
+#  
 #  oe_get("Andorra", extra_tags = c("maxspeed", "oneway", "ref", "junction"), quiet = FALSE)
 #  oe_get("Andora", stringsAsFactors = FALSE, quiet = TRUE, as_tibble = TRUE) # like read_sf
 #  
 #  # Geocode the capital of Goa, India
 #  (geocode_panaji = tmaptools::geocode_OSM("Panaji, India"))
-#  oe_get(geocode_panaji$coords, quiet = FALSE)
+#  oe_get(geocode_panaji$coords, quiet = FALSE) # Large file
 #  oe_get(geocode_panaji$coords, provider = "bbbike", quiet = FALSE)
 #  oe_get(geocode_panaji$coords, provider = "openstreetmap_fr", quiet = FALSE)
 #  
@@ -228,53 +229,38 @@ oe_get("ITS Leeds", provider = "test", quiet = FALSE, force_vectortranslate = TR
 oe_get("ITS Leeds", provider = "test", osmconf_ini = temp_ini, quiet = FALSE, extra_tags = "foot")
 
 ## -------------------------------------------------------------------------------------------------
-my_vectortranslate = c(
-  "-f", "GPKG", #output file format
-  "-overwrite", # overwrite an existing layer
-  "-lco", "GEOMETRY_NAME=geometry", # layer creation options, 
-  "-t_srs", "EPSG:27700", # British National Grid CRS
-  "lines" #layer
-)
 # Check the CRS
-oe_get("ITS Leeds", vectortranslate_options = my_vectortranslate, quiet = FALSE)
+oe_get("ITS Leeds", vectortranslate_options = c("-t_srs", "EPSG:27700"), quiet = FALSE)
 
 ## -------------------------------------------------------------------------------------------------
 my_vectortranslate = c(
-  "-f", "GPKG", # specify the output file format
-  "-overwrite", # overwrite an existing layer
-  "-lco", "GEOMETRY_NAME=geometry", # layer creation options,
-  "-t_srs", "EPSG:27700", # British National Grid CRS
+  "-t_srs", "EPSG:27700", 
   # SQL-like query where we select only the following fields
   "-select", "osm_id, highway", 
-  # SQL-like query where we select only the features where highway is in (footway, cycleway)
-  "-where", "highway IN ('footway', 'cycleway')",
-  "lines" # we want to read-in the lines layer
+  # SQL-like query where we filter only the features where highway is equal to footway or cycleway
+  "-where", "highway IN ('footway', 'cycleway')"
 )
 
 ## -------------------------------------------------------------------------------------------------
 its_leeds = oe_get("ITS Leeds", vectortranslate_options = my_vectortranslate, quiet = FALSE)
 
 ## ---- eval = FALSE--------------------------------------------------------------------------------
-#  # 1. download the data and skip gpkg conversion
+#  # 1. Download the data and skip gpkg conversion
 #  oe_get("Portugal", download_only = TRUE, skip_vectortranslate = TRUE)
 #  
 #  # 2. Define the vectortranslate options
 #  my_vectortranslate = c(
-#    "-f", "GPKG", #output file format
-#    "-overwrite", # overwrite an existing file
-#    "-lco", "GEOMETRY_NAME=geometry", # layer creation options,
 #    # SQL-like query where we select only the features where highway in (primary, secondary, tertiary)
 #    "-select", "osm_id, highway",
-#    "-where", "highway IN ('primary', 'secondary', 'tertiary')",
-#    "lines" #layer
+#    "-where", "highway IN ('primary', 'secondary', 'tertiary')"
 #  )
 #  
 #  # 3. Convert and read-in
 #  system.time({
 #    portugal1 = oe_get("Portugal", vectortranslate_options = my_vectortranslate)
 #  })
-#  #>   user  system elapsed
-#  #>  26.77    9.53   35.18
+#    #  user  system elapsed
+#    # 17.39    9.93   25.53
 
 ## ---- eval = FALSE--------------------------------------------------------------------------------
 #  system.time({
@@ -283,20 +269,17 @@ its_leeds = oe_get("ITS Leeds", vectortranslate_options = my_vectortranslate, qu
 #      dplyr::select(osm_id, highway) %>%
 #      dplyr::filter(highway %in% c('primary', 'secondary', 'tertiary'))
 #  })
-#  #>   user  system elapsed
-#  #> 144.12   26.40  175.97
+#   #   user  system elapsed
+#   # 131.05   28.70  177.03
 #  
 #  nrow(portugal1) == nrow(portugal2)
 #  #> TRUE
 
 ## -------------------------------------------------------------------------------------------------
-my_vectortranslate = c(
-  "-f", "GPKG",
-  "-overwrite",
-  "-spat", c(-1.559184 , 53.807739 , -1.557375 , 53.808094 ),
-  "lines"
-)
-its_small = oe_get ("ITS Leeds", vectortranslate_options = my_vectortranslate)
+its_bbox = st_bbox(c(xmin = -1.559184 , ymin = 53.807739 , xmax = -1.557375 , ymax = 53.808094), crs = 4326) %>% 
+  st_as_sfc()
+
+its_small = oe_get ("ITS Leeds", boundary = its_bbox)
 
 ## ---- echo = FALSE, out.width="85%"---------------------------------------------------------------
 its_leeds = oe_get("ITS Leeds", force_vectortranslate = TRUE, quiet = TRUE)
@@ -313,39 +296,38 @@ plot(st_as_sfc(st_bbox(c(xmin = -1.559184 , ymin = 53.807739 , xmax = -1.557375
 #  
 #  # 2. Define the vectortranslate options
 #  my_vectortranslate = c(
-#    "-f", "GPKG",
-#    "-overwrite",
 #    "-t_srs", "EPSG:32633",
 #    "-select", "highway",
 #    "-where", "highway IN ('primary', 'secondary', 'tertiary', 'unclassified')",
-#    "-clipdst", st_as_text(la_valletta),
-#    "-nlt", "PROMOTE_TO_MULTI",
-#    "lines"
+#    "-nlt", "PROMOTE_TO_MULTI"
 #  )
 #  
-#  # 3. Read-in data
+#  # 3. Download data
+#  oe_get("Malta", skip_vectortranslate = TRUE, download_only = TRUE)
+#  
+#  # 4. Read-in data
 #  system.time({
-#    oe_get("Malta", vectortranslate_options = my_vectortranslate)
+#    oe_get("Malta", vectortranslate_options = my_vectortranslate, boundary = la_valletta, boundary_type = "clipsrc")
 #  })
-#  #> The input place was matched with: Malta
-#  #> The chosen file was already detected in the download directory. Skip downloading.
-#  #> Start with the vectortranslate operations on the input file!
-#  #> 0...10...20...30...40...50...60...70...80...90...100 - done.
-#  #> Finished the vectortranslate operations on the input file!
-#  #> Reading layer `lines' from data source `...' using driver `GPKG'
-#  #> Simple feature collection with 1206 features and 1 field
-#  #> geometry type:  MULTILINESTRING
-#  #> dimension:      XY
-#  #> bbox:           xmin: 451113.3 ymin: 3967859 xmax: 460364.8 ymax: 3976642
-#  #> projected CRS:  WGS 84 / UTM zone 33N
-#  #>   user  system elapsed
-#  #>   1.87    0.85    3.47
+#  # The input place was matched with: Malta
+#  # The chosen file was already detected in the download directory. Skip downloading.
+#  # Start with the vectortranslate operations on the input file!
+#  # 0...10...20...30...40...50...60...70...80...90...100 - done.
+#  # Finished the vectortranslate operations on the input file!
+#  # Reading layer `lines' from data source `C:\Users\Utente\AppData\Local\Temp\RtmpYVijx8\geofabrik_malta-latest.gpkg' using driver `GPKG'
+#  # Simple feature collection with 1205 features and 1 field
+#  # Geometry type: MULTILINESTRING
+#  # Dimension:     XY
+#  # Bounding box:  xmin: 451113.7 ymin: 3967858 xmax: 460364.8 ymax: 3976642
+#  # Projected CRS: WGS 84 / UTM zone 33N
+#  #    user  system elapsed
+#  #    0.55    0.11    0.61
 
 ## ---- echo = FALSE, eval = FALSE------------------------------------------------------------------
 #  malta_regular = oe_get("Malta", force_vectortranslate = TRUE) %>%
 #    dplyr::filter(highway %in% c('primary', 'secondary', 'tertiary', 'unclassified')) %>%
 #    st_transform(32633)
-#  malta_small = oe_get("Malta", vectortranslate_options = my_vectortranslate)
+#  malta_small = oe_get("Malta", vectortranslate_options = my_vectortranslate, boundary = la_valletta, boundary_type = "clipsrc")
 #  
 #  par(mar = rep(0.1, 4))
 #  plot(st_geometry(malta_regular), col = "grey", reset = FALSE)
@@ -357,11 +339,13 @@ knitr::include_graphics("../man/figures/104240598-9d6fb400-545c-11eb-93b5-356390
 
 ## ---- eval = FALSE--------------------------------------------------------------------------------
 #  system.time({
-#    malta_crop = oe_get("Malta", force_vectortranslate = TRUE, force_download = TRUE) %>%
+#    malta_crop = oe_get("Malta", force_vectortranslate = TRUE) %>%
 #      dplyr::filter(highway %in% c('primary', 'secondary', 'tertiary', 'unclassified')) %>%
 #      st_transform(32633) %>%
 #      st_crop(la_valletta)
 #  })
+#  #> user  system elapsed
+#  #> 4.61    1.67    7.69
 
 ## ---- echo = FALSE, eval = FALSE------------------------------------------------------------------
 #  malta_regular = oe_get("Malta", force_vectortranslate = TRUE) %>%
@@ -401,10 +385,12 @@ knitr::include_graphics("../man/figures/104243054-4966ce80-5460-11eb-951b-ca1ce9
 
 ## -------------------------------------------------------------------------------------------------
 # No extra tag
-colnames(oe_get("ITS Leeds"))
+colnames(oe_get("ITS Leeds", quiet = TRUE))
 
-# One extra tag
-oe_get_keys(oe_get("ITS Leeds", provider = "test", download_only = TRUE))
+# Check extra tags
+oe_get_keys("ITS Leeds")
+
+# Add extra tag
 colnames(oe_get(
   "ITS Leeds", 
   provider = "test", 
