@@ -1,5 +1,9 @@
 test_that("oe_update(): simplest example works", {
+  # I always need internet connection when running oe_update()
+  skip_on_cran()
   skip_if_offline()
+
+
   out = oe_get(
     "ITS Leeds",
     provider = "test",
@@ -8,9 +12,9 @@ test_that("oe_update(): simplest example works", {
     quiet = TRUE
   )
   expect_error(oe_update(tempdir(), quiet = TRUE), NA)
+
   # AG: I decided to comment out that test since I don't see any benefit testing
   # the "verbose" output during R CMD checks (that I rarely check manually)
   # expect_message(oe_update(fake_dir, quiet = FALSE))
-
-  file.remove(oe_find("ITS Leeds", download_directory = tempdir()))
+  file.remove(list.files(tempdir(), pattern = "its-example", full.names = TRUE))
 })
